@@ -68,6 +68,8 @@ class XNetOptionParser(optparse.OptionParser):
                         help='time interval between each action ')
         self.add_option('', '--repeat', dest='repeat', metavar='ntimes',
                         help='repeat action this many times (-1 = infinity)')
+        self.add_option('', '--time', dest='time', action='store_true',
+                        help='print time consumed by action')
         self.add_option('-T', '--supress-timeout', dest='supress_timeout',
                         help='supress messages from tasks timing out', action='store_true')
         self.add_option('-f', '--format', dest='format', metavar='fmt',
@@ -160,10 +162,7 @@ def _get_output_filepath(a, base):
     # which should not affect the file system path
     # where the output file is created.
     #
-    item = str(a._item).strip()
-    item = item.replace('/', '__')
-    item = item.replace('\\', '--')
-    path = base + item
+    path = base + a.__filename__()
     return path
 
 
@@ -265,6 +264,11 @@ def _run(ToolImplementation, tool_reads_stdin):
     if opt.split_tee:
         opt.split_output = opt.split_tee
 
+    #
+    # verify commandline options and do preparations
+    #
+    ToolImplementation.__setup__(opt)
+
     pool = gevent.pool.Pool(_parallelism)
     actions = []  # all spawned actions
     greenlets = []
@@ -282,7 +286,7 @@ def _run(ToolImplementation, tool_reads_stdin):
             gevent.kill(g, xnet.tools.WaitTimeout)
             g.join()
 
-    inputchain = ToolImplementation.__massage__(inputchain)
+    inputchain = ToolImplementation.__massage__(inputchain, opt)
 
     kwargs = {}
 
@@ -297,7 +301,6 @@ def _run(ToolImplementation, tool_reads_stdin):
         inputchain = repeaterator(inputchain, _repeat)
 
     for (i, line) in enumerate(inputchain):
-        line = line.strip()
         action = ToolImplementation(opt, **kwargs)
         pool.wait_available()
         if _wait:
@@ -357,29 +360,30 @@ def output_action_result(a, opt, _outfile):
         path = _get_output_filepath(a, opt.split_output)
         outfile = open(path, 'wb')
     if not a.error is None:
-        print_error(a)
-    elif not a.finished:
-        if not opt.supress_timeout:
-            msg = a.__timeout__()
+        if a.error == xnet.tools.WaitTimeout:
+            if not opt.supress_timeout:
+                #
+                # __timeout__() messages always go to stderr.
+                #
+                msg = a.__timeout__()
+                if not msg is None:
+                    sys.stderr.write(msg)
+        else:
+            print_error(a)
+    else:
+        if a._action_result:
+            msg = str(a)
+            #
+            # ignore empty output from Tool.__format__()
+            #
+            if len(msg) == 0:
+                return
+            msg += '\n'
             if a.stderr:
                 outfile = sys.stderr
-            if not msg is None:
-                outfile.write(msg)
-                if opt.split_tee and not a.stderr:
-                    _outfile.write(msg)
-    else:
-        msg = str(a)
-        #
-        # ignore empty output from Tool.__format__()
-        #
-        if len(msg) == 0:
-            return
-        msg += '\n'
-        if a.stderr:
-            outfile = sys.stderr
-        outfile.write(msg)
-        if opt.split_tee and not a.stderr:
-            _outfile.write(msg)
+            outfile.write(msg)
+            if opt.split_tee and not a.stderr:
+                _outfile.write(msg)
 
     _outfile.flush()
     if not outfile is _outfile:
