@@ -44,7 +44,7 @@ class SSLInfoTool(Tool):
         cls._print_source(__file__)
 
     @classmethod
-    def __massage__(self, iterator):
+    def __massage__(self, iterator, options):
         for item in iterator:
             item = item.strip()
             try:
@@ -167,6 +167,7 @@ Format variables available for %s.
  OU          - subject organizational unit
  CN          - subject common name
  email       - subject email address
+ time        - time consumed by action, in ms
         ''' % cls.__toolname__
         return output
 
