@@ -13,14 +13,17 @@
 __all__ = ['WebgetTool']
 
 
-import unittest
-import doctest
+#import unittest
+#import doctest
 import sys
 import os
 import codecs
 import urlparse
 
 from xnet.tools import Tool
+from xnet.net.ipv4 import PortRangeIterator
+from xnet.net.http import UrlIterator
+#from xnet.tools import ToolMixin_AddrIterator
 #from xnet.debug import pdb
 import xnet.packages.urllib3
 import xnet.packages.urllib3.exceptions
@@ -37,6 +40,8 @@ class WebgetTool(Tool):
     __description__ = 'get information from web services'
 
     cmdline_options = [
+        ('-p', '--ports', 'use these ports for urls lacking port AND protocol components',
+            dict(dest='ports', metavar='m-n,..')),
         ('-s', '--server', 'print server header',
             dict(dest='server', action='store_true')),
         ('-S', '--https', 'default to https when protocol is missing',
@@ -56,9 +61,11 @@ class WebgetTool(Tool):
         ('', '--xpath', 'grab elements matching xpath expression',
             dict(action='append', metavar='expr')),
         ('', '--xpath-delimiter', 'xpath output delimiter when using multiple --xpath\'s',
-            dict(dest='xpath_delimiter', action='store_true')),
+            dict(dest='xpath_delimiter', metavar='delim')),
         ('', '--xpath-strict', 'exit if not all --xpath\'s are matched',
             dict(dest='xpath_strict', action='store_true')),
+        ('', '--tag-lines', 'prefix each line with current url',
+            dict(dest='tag_lines', action='store_true')),
     ]
 
     @classmethod
@@ -94,31 +101,31 @@ class WebgetTool(Tool):
             try:
                 result = self._http.request(*args, **kw)
             except xnet.packages.urllib3.exceptions.ClosedPoolError:
-                pass
+                pass  # urllib3 seems to take care of this
             else:
                 done = True
         return result
 
-    def __parse__(self, line, iterator):
-        '''
-            only accepts plain urls atm.
-        '''
-        result = {}
-        self._url = url = line.strip()
-        if not '://' in url:
-            proto = 'http'
-            if self.options.https:
-                proto = 'https'
-            url = proto + '://' + url
-        result['url'] = url
-        result['host'] = self.options.host or self._get_host(url)
-        return result
+    @classmethod
+    def __massage__(cls, iterator, options):
+        default_scheme = 'http'
+        default_ports = [80]
+        if options.https:
+            default_scheme = 'https'
+            default_ports = [443]
+        if options.ports:
+            default_ports = list(PortRangeIterator(options.ports))
+        urlIterator = UrlIterator(default_scheme=default_scheme, default_ports=default_ports)
+        for xurl in iterator:
+            for url in urlIterator.expand(xurl):
+                yield url
 
-    def _get_host(self, url):
-        host = url.split('://', 1)[1]
-        if '/' in host:
-            host = host.split('/', 1)[0]
-        return host
+    def __parse__(self, url, iterator):
+        url = url.strip()
+        result = {}
+        result['url'] = url
+        result['host'] = self._get_host(url)
+        return result
 
     def _explicitly_allowed_redirect(self, ourl, lurl):
         if lurl.netloc.lower() in self._redirect_whitelist:
@@ -134,10 +141,13 @@ class WebgetTool(Tool):
                         return True
         return False
 
+    def _get_host(self, url):
+        return urlparse.urlparse(url).netloc
+
     def __action__(self, parse_result):
         result = {}
-        _host = parse_result['host']
-        _url = parse_result['url']
+        _url = self._url = parse_result['url']
+        _host = self.options.host or self._get_host(_url)
         _method = self.options.method or 'GET'
         _response = None
         _body_unicode = None
@@ -253,9 +263,13 @@ class WebgetTool(Tool):
         if charset:
             _body_unicode = codecs.decode(_body, charset, 'ignore')
         else:
-            _body_unicode = codecs.decode(_body, 'Latin-1')
+            _body_unicode = codecs.decode(_body, 'Latin-1', 'ignore')
             #_body_unicode = unicode(_body)
 
+        if read_response and hasattr(self.options, 'tag_lines') and self.options.tag_lines:
+            tag = self._url + ':'
+            _body = '\n' + tag + _body.replace('\n', '\n' + tag)
+
         result['host'] = _host
         result['url'] = _url
         result['response'] = _response
@@ -334,6 +348,7 @@ Format variables for %s:
  code     - status code
  msg      - status message
  body     - response body
+ time     - time consumed by action, in ms
 
 Default format depend on commandline arguments.
         ''' % cls.__toolname__
